;**********************************************************************
;    Filename:	    BigC8_05.asm                                      *
;    Date:          Dienstag, 22. September 2015, 16:15:15            *
;    File Version:                                                    *
;                                                                     *
;    Author:        ffje                                              *
;    Company:       fE                                                *
;                                                                     *
;**********************************************************************
;    Files required:                                                  *
;                    16F628A_01.lkr                                   *
;                                                                     *
;**********************************************************************
;    Notes:                                                           *
;        1. neue LCD-Ausgabe fuer weniger Programm-Speicher           *
;                                                                     *
;**********************************************************************

	list      p=16F628A           ; list directive to define processor
	#include <p16F628A.inc>       ; processor specific variable definitions

	errorlevel  -302              ; suppress message 302 from list file

	__CONFIG   _CP_OFF & DATA_CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_OFF & _INTOSC_OSC_NOCLKOUT

;***** DEFINITIONS
#define	p_CLK	PORTA,	6	; LCD-Clock
#define	p_DIN	PORTA,	7	; LCD-Data
#define	p_Dc	PORTB,	6	; LCD: H = RAM-data, L = command
#define	p_ce	PORTB,	4	; LCD: H = init Ser.-Interface
#define	p_res	PORTB,	7	; LCD: L = reset LCD

#define	p_SWIN	PORTB,	0	; On/Off-input
#define	p_RX	PORTB,	1	; Serial-Port-input (TX = RB2)
#define	p_SWOUT	PORTB,	3	; On/Off-output
#define	p_CHARG	PORTB,	5	; Elko-charge-output

#define	F_ovr	Flags0,	0	; C-Value-Ueberlauf
#define	F_noC	Flags0,	1	; C-Value-Unterlauf
#define	F_Hovr	Flags0,	2	; C-Value-Ueberlauf history
#define	F_HnoC	Flags0,	3	; C-Value-Unterlauf history
#define	F_HLB0	Flags0,	4	; Zeilen-Anzahl-Bit0 (history)
#define	F_HLB1	Flags0,	5	; Zeilen-Anzahl-Bit1 (history)
#define	F_res6	Flags0,	6	; Universal-Flag
#define	F_res7	Flags0,	7	; TMR1-Ints nicht verarbeiten

;***** VARIABLE DEFINITIONS
; shared uninitialized Data
		UDATA_SHR	0x70
w_temp		RES     1		; wreg context saving 
s_temp		RES     1		; status context saving
Flags0		RES		1		; Flags0

; unshared in Bank0
			udata	0x20
C_Val_L		RES		1		; C-Value lowByte
C_Val_H		RES		1		; C-Value highByte
C_oVal_L	RES		1		; old-C-Value lowByte (history)
C_oVal_H	RES		1		; old-C-Value highByte (history)

wg100ms		RES		1		; temp Zaehler fuer 100 ms
wg1sec		RES		1		; temp Zaehler fuer 1 sec
T10m_a		RES		1		; 10 ms Zaehler_a
T10m_b		RES		1		; 10 ms Zaehler_b
T100m_a		RES		1		; 100 ms Zaehler_a
T1s_a		RES		1		; 1 sec Zaehler_a

LCD_Byte	RES		1		; geht bitweise zum LCD
CusorX		RES		1		; Cusor-Position-X
CusorY		RES		1		; Cusor-Position-Y (0..4)
Zeichen		RES		1		; aus Zeichen-Tabelle zum LCD
Offset		RES		1		; Abstand von Basis der Zeichen-Tabelle
OU_Cnt		RES		1		; oberer/unterer Teil eines Zeichens (LCD)
ColmCnt		RES		1		; Spalte eines Zeichens (LCD)
PCtxtGr		RES		1		; Groesse eines Textes fuer PC-Ser-Port
PCtxtPos	RES		1		; Abstand von Basis der PC-Texte-Tabelle
PC_L		RES		1		; temp. Var fuer Table-Read
PC_H		RES		1		; temp. Var fuer Table-Read
Anz0n		RES		1		; Anzahl fuehrender Nullen

BCDvalH		RES		1		; 10000/1000-er des Elko-Wertes
BCDvalM		RES		1		; 100/10-er des Elko-Wertes
BCDvalL		RES		1		; 1-er des Elko-Wertes
count		RES		1		; temp. Var f. Bin-to-BCD
Temp_a		RES		1		; temp. Var f. Bin-to-BCD und ClrDepLns
Temp_m      RES     1       ; temp. Var f. multipl. mit konst. 20
H_byte      RES     1       ; High byte of the 16 bit result
L_byte      RES     1       ; Low byte of the 16 bit result

;**********************************************************************
RESET_VECTOR	CODE	0x000	; processor reset vector
	goto    main			; go to beginning of program
;**********************************************************************
INT_VECTOR	CODE	0x004	; interrupt vector location
Int							; wreg u. STATUS retten, Banks = 0
	movwf   w_temp			; save wreg
	swapf	STATUS, W		; move status into wreg
	movwf	s_temp			; save STATUS
	clrf	STATUS			; Banks = 0
;*********************************
IF_Test						; Int-Ausloeser testen
	banksel	0x0
	btfsc	INTCON, T0IF
	goto	TMR0_Int		; T0IF ist H
	btfsc	PIR1, TMR1IF
	goto	TMR1_Int		; TMR1IF ist H
;*********************************
Int_Ende					; STATUS u. wreg zurueck
	swapf	s_temp, W		; STATUS zurueck
	movwf	STATUS
	swapf	w_temp, F		; wreg zurueck
	swapf	w_temp, W
	retfie					; return from interrupt
;*********************************
TMR0_Int
	movlw	d'101'			; 256 - ca. 170,7 microsec
	movwf	TMR0
	nop
	bcf		F_noC			; reset noC-Flag
	bcf		INTCON, T0IF	; reset TMR0_IntFlag
inc_C_Value
	incfsz	C_Val_L, F		; inc 16-Bit C-Value
	goto	IF_Test
	incfsz	C_Val_H, F
	goto	IF_Test
	bsf		F_ovr			; C-Value-Ueberlauf
	btfsc	F_res7			; TMR1-Ints testen
	goto	Int_Ende		; TMR1-Ints nicht verarbeiten
	goto	IF_Test
;*********************************
TMR1_Int
	bcf		T1CON, TMR1ON
	movlw	0x20			; 65536 - 1250 = 64286 (ungefaehr)
	movwf	TMR1L			; fuer alle 10msec
	movlw	0xFB
	movwf	TMR1H
	bsf		T1CON, TMR1ON	; TMR1 neu starten
jede10ms					; was alle 10ms gemacht wird...
	incf	T10m_a, F
	btfsc	p_SWIN
	goto	no_p_SWIN_prsd	; Taster nicht gedrueckt (p_SWIN = H)
	decfsz	T10m_b, F		; Taster-gedrueckt-Zeit auswerten
	goto	Test100ms		; Zeit zu gering, pwr-off-Timer erhalten
; ausschalten vorbereiten
	bcf		INTCON, GIE		; disable global Ints
	bcf		p_CHARG			; RB5 auf L (C entladen...)
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	bcf		TRISB, 5		; RB5 auf Low_Z
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
	bsf		p_ce			; LCD-Ser-Port-Init
	goto	Msg_pwr_down	; unbedingt ausschalten
no_p_SWIN_prsd
	clrf	T10m_b			; reset pwr-off-Timer wenn Taster offen
;*********************************
Test100ms
	decfsz	wg100ms, F
	goto	Test1Sec
	movlw	d'10'			; 100ms TempVar neu setzen
	movwf	wg100ms
jede100ms					; was alle 100ms gemacht wird...
	incf	T100m_a, F
;*********************************
Test1Sec
	decfsz	wg1sec, F
	goto	TMR1_Int_Ende
	movlw	d'100'			; 1sec TempVar neu setzen
	movwf	wg1sec
jedeSec						; was jede Sekunde gemacht wird...
	incf	T1s_a, F
;*********************************
TMR1_Int_Ende
	bcf		PIR1, TMR1IF	; reset TMR1_IntFlag	
	goto	IF_Test
;**********************************************************************
main						; remaining code goes here
Osc_Init
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	bsf		PCON, OSCF		; Oszillator-Freq auf 4 MHz
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
;*********************************
Ports_Init
	clrf	PORTA
	movlw	b'00011000'		; p_SWOUT, p_ce = H
	movwf	PORTB
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	movlw	b'00001111'		; RB7..6 lcd, RB3..0 comparator
	movwf	TRISA
	movlw	b'00000011'		; RB7..6,4 lcd, RB5 Charge
	movwf	TRISB			; RB3 p_SWOUT, RB2..1 Ser, RB0 p_SWIN
	bcf		OPTION_REG,	NOT_RBPU	; week-pull-up aktiv
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
;*********************************
Interrupts_enable
	clrf	T10m_b			; reset Taster-gedrueckt-Zeit
	bsf		INTCON, GIE		; enable global Ints
	bsf		INTCON, PEIE	; enable peripheral Ints
;*********************************
Comp_Init
	movlw	b'00000010'		; int.Ref., no invert.,
	movwf	CMCON			; C1Vin- = RA0, C2Vin- = RA1
;*********************************
Timer1_Init					; 10ms, 100ms, 1s -Ticks
	movlw	0x1E			; 65536 - 1250 = 64286
	movwf	TMR1L			; fuer alle 10ms
	movlw	0xFB
	movwf	TMR1H
	movlw	d'10'			; 100ms TempVar vorbelegen
	movwf	wg100ms
	movlw	d'100'			; 1sec TempVar vorbelegen
	movwf	wg1sec
	movlw	b'00110001'		; prescale 8, T1osc off, no sync,
	movwf	T1CON			; int.clock, enable Timer1
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	bsf		PIE1, TMR1IE	; enable Timer1Int
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
;*********************************
LCD_reset					; p_res muss 150ms auf L bleiben
	clrf	T10m_a
	movf	T10m_a, W
	sublw	d'14'			; 150ms = (14 + 1) * 10ms
	btfsc	STATUS, C
	goto	$-3				; Ergebnis ist nicht negativ
	bsf		p_res			; p_res = H (LCD-reset fertig)
;*********************************
LCD_init
	movlw	0x21			; activate chip and H = 1
	call	wrComm
	movlw	0xC2			; set LCD-voltage to about 7V (0xC2)
	call	wrComm          ; mein dunkleres LCD braucht 7,6V (0xCC)
	movlw	0x13			; adjust voltage-bias
	call	wrComm
	movlw	0x20			; horizontal adressing H = 0
	call	wrComm
	movlw	0x09			; activate all segments
	call	wrComm
	call	clr3DZ			; erase all pixel on DDRAM
	movlw	0x08			; blank the display
	call	wrComm
	movlw	0x0C			; display normal
	call	wrComm
	clrf	CusorX			; cusor home
	clrf	CusorY
	call	setCusor
;*********************************
USART_Init
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	bsf		TXSTA, BRGH		; high-speed
	movlw	0x19			; 9600 baud
	movwf	SPBRG
	bcf		TXSTA, SYNC		; enable async-ser-port
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
	bsf		RCSTA, SPEN		; enable Seriell-Port
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	bcf		TXSTA, TX9		; 8 Bit
	bsf		TXSTA, TXEN		; enable Transmission
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
;*********************************
badPwrUpTest				; p_SWIN muss ca. 30 ms auf L sein
	clrf	T10m_a
	bsf		F_res6
	btfsc	p_SWIN			; p_SWIN ist bei Taster L
	bcf		F_res6			; clr Flag, wenn Taster nicht gedrueckt
	movf	T10m_a, W
	sublw	d'2'			; 30ms = (2 + 1) * 10ms
	btfsc	STATUS, C
	goto	$-5				; weiter testen
	btfss	F_res6
	goto	hw_off_fast		; wenn Taster nicht immer gedrueckt war
;*********************************
; Batterie-Test
	call	Uref_2_7625V	; SpgRef auf 2,7625V fuer Batterie-Test
	btfsc	CMCON, C2OUT	; C2OUT ist bei LowBatt gesetzt
	goto	lowBatt_PD
;*********************************
; Taster-Up-Wait
Msg_on						; Geraet-EIN-Meldung
	call	ClrDepLns		; erst LCD-Zeilen loeschen
	movlw	d'25'			; xy-Pos einstellen
	movwf	CusorX
	movlw	d'2'
	movwf	CusorY
	call	setCusor
;*********************************
	movlw	0x1A			; o
	movwf	Zeichen
	call	wrChar
	movlw	0x0A			; LeerZeichen
	movwf	Zeichen
	call	wrChar
	movlw	0x13			; n
	movwf	Zeichen
	call	wrChar
; dies war ein 1-Zeiler
	bsf		F_HLB0
	bcf		F_HLB1
;*********************************
; ist mit PC verbunden?
	btfss	p_RX
	goto	p_SWIN_30ms_wait
	movlw	d'4'			; mit PC verbunden
	movwf	PCtxtGr
	movlw	HIGH PC_Text_01
	movwf	PC_H
	movlw	LOW PC_Text_01
	movwf	PC_L
	call	ReadTabPC
	clrf	T10m_b			; reset Taster-gedrueckt-Zeit
;*********************************
p_SWIN_30ms_wait			; p_SWIN muss ca. 30 ms auf H sein
	clrf	T10m_a
	bsf		F_res6
	btfss	p_SWIN			; p_SWIN ist bei Taster L
	bcf		F_res6			; clr Flag, wenn Taster gedrueckt
	movf	T10m_a, W
	sublw	d'2'			; 30ms = (2 + 1) * 10ms
	btfsc	STATUS, C
	goto	$-5				; weiter testen
	btfss	F_res6
	goto	p_SWIN_30ms_wait; wenn Taster nicht immer offen war
;*********************************
clr_aPD_Time				; auto-power-off abgebrochen
	clrf	T1s_a			; clr auto_power_off_Time
	movlw	b'00110000'		; set Flags0 auf 3 DoppelZeilen loeschen 
	movwf	Flags0
	movlw	0xFF			; oldVals-Init
	movwf	C_oVal_L
	movwf	C_oVal_H
;*********************************
clr_H_exceptns
	bcf		F_Hovr			; reset history-ovr-Flag
	bcf		F_HnoC			; reset history-noC-Flag
;*********************************
ElkoTest
	clrf	C_Val_L			; reset C-Value
	clrf	C_Val_H
; init ovr & noC Flags
	bcf		F_ovr			; reset ovr-Flag
	bsf		F_noC			; set noC-Flag
; auto-power-off-timer testen
	movf	T1s_a, W
	sublw	d'119'			; 120s = (119 + 1) * 1s
	btfsc	STATUS, C
	goto	C_entladen		; weil noch kein auto-power-off
Msg_auto_PD
	call	ClrDepLns		; erst LCD-Zeilen loeschen
	movlw	d'15'			; xy-Pos einstellen fuer 1. Zeile
	movwf	CusorX
	clrf	CusorY
	call	setCusor
;*********************************
	movlw	0x15			; a
	movwf	Zeichen
	call	wrChar
	movlw	0x1E			; u
	movwf	Zeichen
	call	wrChar
	movlw	0x1D			; t
	movwf	Zeichen
	call	wrChar
	movlw	0x1A			; o
	movwf	Zeichen
	call	wrChar
;*********************************
	movlw	d'10'			; ; xy-Pos einstellen fuer 2. Zeile
	movwf	CusorX
	movlw	d'2'
	movwf	CusorY
	call	setCusor
;*********************************
	movlw	0x1B			; p
	movwf	Zeichen
	call	wrChar
	movlw	0x1A			; o
	movwf	Zeichen
	call	wrChar
	movlw	0x1F			; w
	movwf	Zeichen
	call	wrChar
	movlw	0x17			; e
	movwf	Zeichen
	call	wrChar
	movlw	0x1C			; r
	movwf	Zeichen
	call	wrChar
;*********************************
	movlw	d'20'			; ; xy-Pos einstellen fuer 3. Zeile
	movwf	CusorX
	movlw	d'4'
	movwf	CusorY
	call	setCusor
;*********************************
	movlw	0x1A			; o
	movwf	Zeichen
	call	wrChar
	movlw	0x18			; f
	movwf	Zeichen
	call	wrChar
	movlw	0x18			; f
	movwf	Zeichen
	call	wrChar
; dies war ein 3-Zeiler
	bsf		F_HLB0
	bsf		F_HLB1
;*********************************
; ist mit PC verbunden?
	btfss	p_RX
	goto	aPD_Msg_3s_wait
	movlw	d'16'			; mit PC verbunden
	movwf	PCtxtGr
	movlw	HIGH PC_Text_04
	movwf	PC_H
	movlw	LOW PC_Text_04
	movwf	PC_L
	call	ReadTabPC
;*********************************
aPD_Msg_3s_wait				; 3s auf auto-PD-Abbruch warten
	clrf	T100m_a			; auto-power-off-Anzeige-Zeit
	btfss	p_SWIN			; p_SWIN ist bei Taster L
	goto	clr_aPD_Time	; auto-power-off abgebrochen
	movf	T100m_a, W
	sublw	d'29'			; 3s = (29 + 1) * 100ms
	btfsc	STATUS, C
	goto	$-5				; Ergebnis ist nicht negativ
	goto	hw_off
;*********************************
C_entladen
	call	C_Entladung		; ...return mit RB5 = Low_Z
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	bsf		TRISB, 5		; RB5 High_Z
C_aufladen					; zaehlt C-Value, zeigt ueber/unter-Lauf
	call	Uref_2_7625V	; SpgRef auf 2,7625V  ---return mit Bank0
	bsf		p_CHARG			; RB5 auf H
TMR0_Init					; Zeit / Interrupt u. p_CHARG aktivieren
	movlw	d'100'			; 256 - ca. 170,7 microsec
	movwf	TMR0
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	bcf		OPTION_REG, T0CS; TMR0-Clk-Source = Osc/4  noPreScaler
; charge C
	bcf		TRISB, 5		; RB5 Low_Z  (startet Elko aufladen)
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
	bcf		INTCON, T0IF	; reset TMR0_IntFlag
	bsf		INTCON,	T0IE	; TMR0-Int-enable
Uc_zu_wenig					; Uc am Comp. C1
	btfsc	F_ovr			; Ueberlauf testen
	goto	TMR0_disable
; Comparator C2, wg. LowBatt-Test
	btfsc	CMCON, C2OUT	; C2OUT ist bei LowBatt gesetzt
	goto	lowBatt_PD
; Comparator C1, wg. C-Aufladung
	btfsc	CMCON, C1OUT
	goto	Uc_zu_wenig		; Uc ist kleiner als 2,7625V
TMR0_disable				; keine TMR0-Ints u. RB5 auf High_Z
	bcf		INTCON, T0IE	; TMR0-Int-disable
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	bsf		OPTION_REG,	T0CS; Timer0 als counter
	bsf		TRISB, 5		; RB5 High_Z
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
exceptn_ovr					; exception Ueberlauf
	btfss	F_ovr			; Ueberlauf testen
	goto	exceptn_noC		; um auf Unterlauf zu tesetn / reagieren
	btfsc	F_Hovr			; history-Ueberlauf testen
	goto	ElkoTest
;*********************************
Msg_ovr						; Ueberlauf-Meldg (C zu gross / shortcut)
	call	ClrDepLns		; erst LCD-Zeilen loeschen
	movlw	d'5'			; xy-Pos einstellen
	movwf	CusorX
	movlw	d'2'
	movwf	CusorY
	call	setCusor
;*********************************
	movlw	0x0D			; C
	movwf	Zeichen
	call	wrChar
	movlw	0x0E			; >
	movwf	Zeichen
	call	wrChar
	movlw	0x0D			; C
	movwf	Zeichen
	call	wrChar
	movlw	0x12			; m
	movwf	Zeichen
	call	wrChar
	movlw	0x10			; a
	movwf	Zeichen
	call	wrChar
	movlw	0x14			; x
	movwf	Zeichen
	call	wrChar
; dies war ein 1-Zeiler
	bsf		F_HLB0
	bcf		F_HLB1
;*********************************
; ist mit PC verbunden?
	btfss	p_RX
	goto	setMyFlags1
	movlw	d'10'			; mit PC verbunden
	movwf	PCtxtGr
	movlw	HIGH PC_Text_02
	movwf	PC_H
	movlw	LOW PC_Text_02
	movwf	PC_L
	call	ReadTabPC
setMyFlags1
	bsf		F_Hovr			; set history-Ueberlauf
	bcf		F_HnoC			; reset history-no C
	goto	ElkoTest
;*********************************
exceptn_noC
	btfss	F_noC			; auf no C testen
	goto	C_display		; C-Value anzeigen
	btfsc	F_HnoC			; history-no C testen
	goto	ElkoTest
Msg_noC						; Unterlauf-Meldungg (C zu klein)
	call	ClrDepLns		; erst LCD-Zeilen loeschen
	movlw	d'5'			; xy-Pos einstellen
	movwf	CusorX
	movlw	d'2'
	movwf	CusorY
	call	setCusor
;*********************************
	movlw	0x0D			; C
	movwf	Zeichen
	call	wrChar
	movlw	0x0F			; <
	movwf	Zeichen
	call	wrChar
	movlw	0x0D			; C
	movwf	Zeichen
	call	wrChar
	movlw	0x12			; m
	movwf	Zeichen
	call	wrChar
	movlw	0x11			; i
	movwf	Zeichen
	call	wrChar
	movlw	0x13			; n
	movwf	Zeichen
	call	wrChar
; dies war ein 1-Zeiler
	bsf		F_HLB0
	bcf		F_HLB1
;*********************************
; ist mit PC verbunden?
	btfss	p_RX
	goto	setMyFlags2
	movlw	d'10'			; mit PC verbunden
	movwf	PCtxtGr
	movlw	HIGH PC_Text_03
	movwf	PC_H
	movlw	LOW PC_Text_03
	movwf	PC_L
	call	ReadTabPC
;*********************************
setMyFlags2
	bsf		F_HnoC			; set history-no C
	bcf		F_Hovr			; reset history-Ueberlauf
	goto	ElkoTest
;*********************************
C_display					; C-Value anzeigen
	movf	C_oVal_L, W
	xorwf	C_Val_L, W
	btfss	STATUS, Z
	goto	wrNew			; wenn C-Val geaendert, neu berechnen
	movf	C_oVal_H, W
	xorwf	C_Val_H, W
	btfss	STATUS, Z
	goto	wrNew			; wenn C-Val geaendert, neu berechnen
	goto	clr_H_exceptns	; vor ElkoTest, wg. history-Flags
;*********************************
wrNew						; history-C-Value erneuern
	movf	C_Val_L, W
	movwf	C_oVal_L
	movf	C_Val_H, W
	movwf	C_oVal_H
	call	HexBCD			; 16-Bit-Binaer-zu-BCD-Wandlung
;*********************************
; Anzahl fuehrender Nullen ermitteln
	clrf	Anz0n
	movf	BCDvalH, W
	andlw	0x0F
	btfss	STATUS, Z
	goto	Anz0io
	incf	Anz0n, F
	movf	BCDvalM, W
	andlw	0xF0
	btfss	STATUS, Z
	goto	Anz0io
	incf	Anz0n, F
	movf	BCDvalM, W
	andlw	0x0F
	btfss	STATUS, Z
	goto	Anz0io
	incf	Anz0n, F
	movf	BCDvalL, W
	andlw	0xF0
	btfss	STATUS, Z
	goto	Anz0io
	incf	Anz0n, F
	movf	BCDvalL, W
	andlw	0x0F
	btfss	STATUS, Z
	goto	Anz0io
	incf	Anz0n, F
;*********************************
Anz0io						; zum LCD
	call	ClrDepLns		; erst LCD-Zeilen loeschen
	movlw	d'2'			; y-Pos einstellen
	movwf	CusorY
; abhaengig von Anzahl fuehrender Nullen...
	movf	Anz0n, W
	xorlw	0x00
	btfsc	STATUS, Z
	goto	LCD_TenTsnd
	movf	Anz0n, W
	xorlw	0x01
	btfsc	STATUS, Z
	goto	LCD_Tsnd
	movf	Anz0n, W
	xorlw	0x02
	btfsc	STATUS, Z
	goto	LCD_Hndt
	movf	Anz0n, W
	xorlw	0x03
	btfsc	STATUS, Z
	goto	LCD_Ten
	goto	LCD_One
; ...die x-Pos einstellen und Cusor entspr. setzen...
LCD_TenTsnd
	clrf	CusorX
	call	setCusor
	goto	Zahlen5
LCD_Tsnd
	movlw	d'6'
	movwf	CusorX
	call	setCusor
	goto	Zahlen4
LCD_Hndt
	movlw	d'12'
	movwf	CusorX
	call	setCusor
	goto	Zahlen3
LCD_Ten
	movlw	d'18'
	movwf	CusorX
	call	setCusor
	goto	Zahlen2
LCD_One
	movlw	d'24'
	movwf	CusorX
	call	setCusor
	goto	Zahlen1
; ...und die verbleibenden Zahlen (ohne fuehrende Nullen) zum LCD
Zahlen5
	movf	BCDvalH, W
	call	L_nibble
Zahlen4
	swapf	BCDvalM, W
	call	L_nibble
Zahlen3
	movf	BCDvalM, W
	call	L_nibble
Zahlen2
	swapf	BCDvalL, W
	call	L_nibble
Zahlen1
	movf	BCDvalL, W
	call	L_nibble
; ...und die Mass-Einheit microFarad
	movlw	0x0B
	movwf	Zeichen
	call	wrChar
	movlw	0x0C
	movwf	Zeichen
	call	wrChar
; dies war ein 1-Zeiler
	bsf		F_HLB0
	bcf		F_HLB1
;*********************************
; ist mit PC verbunden?
	btfss	p_RX
	goto	clr_H_exceptns	; vor ElkoTest, wg. history-Flags
; mit PC verbunden, abhaengig von Anzahl fuehrender Nullen...
	movf	Anz0n, W
	xorlw	0x00
	btfsc	STATUS, Z
	goto	ZehnTausender
	movf	Anz0n, W
	xorlw	0x01
	btfsc	STATUS, Z
	goto	Tausender
	movf	Anz0n, W
	xorlw	0x02
	btfsc	STATUS, Z
	goto	Hunderter
	movf	Anz0n, W
	xorlw	0x03
	btfsc	STATUS, Z
	goto	Zehner
	goto	Einer
; ...die verbleibenden Zahlen (ohne fuehrende Nullen) zum PC
ZehnTausender
	movf	BCDvalH, W
	call	L_nibble_ascii
Tausender
	swapf	BCDvalM, W
	call	L_nibble_ascii
Hunderter
	movf	BCDvalM, W
	call	L_nibble_ascii
Zehner
	swapf	BCDvalL, W
	call	L_nibble_ascii
Einer
	movf	BCDvalL, W
	call	L_nibble_ascii
; ...und LeerZeichen & F & CR_LF
	movlw	" "
	call	writeSer
	movlw	""				; !!!!! oder "u"
	call	writeSer
	movlw	"F"
	call	writeSer
	movlw	0x0A			; LineFeed
	call	writeSer
	movlw	0x0D			; CarrigeReturn
	call	writeSer
	goto	clr_H_exceptns	; vor ElkoTest, wg. history-Flags
;**********************************************************************
Msg_pwr_down				; !!!!! ab hier besteht Stack-Fehler
	call	ClrDepLns		; erst LCD-Zeilen loeschen
	movlw	d'10'			; xy-Pos einstellen fuer 1. Zeile
	movwf	CusorX
	movlw	d'1'
	movwf	CusorY
	call	setCusor
;*********************************
	movlw	0x1B			; p
	movwf	Zeichen
	call	wrChar
	movlw	0x1A			; o
	movwf	Zeichen
	call	wrChar
	movlw	0x1F			; w
	movwf	Zeichen
	call	wrChar
	movlw	0x17			; e
	movwf	Zeichen
	call	wrChar
	movlw	0x1C			; r
	movwf	Zeichen
	call	wrChar
;*********************************
	movlw	d'20'			; xy-Pos einstellen fuer 2. Zeile
	movwf	CusorX
	movlw	d'3'
	movwf	CusorY
	call	setCusor
;*********************************
	movlw	0x1A			; o
	movwf	Zeichen
	call	wrChar
	movlw	0x18			; f
	movwf	Zeichen
	call	wrChar
	movlw	0x18			; f
	movwf	Zeichen
	call	wrChar
;*********************************
; ist mit PC verbunden?
	btfss	p_RX
	goto	p_SWIN_up_wait                                                    
	movlw	d'11'			; mit PC verbunden
	movwf	PCtxtGr
	movlw	HIGH PC_Text_05
	movwf	PC_H
	movlw	LOW PC_Text_05
	movwf	PC_L
	call	ReadTabPC
;*********************************
p_SWIN_up_wait				; entladen u. Taster 30ms offen warten
	call	C_Entladung		; bis unter 0,425V entladen
p_SWIN_30up_wait			; p_SWIN muss ca. 30 ms auf H sein
	clrf	T1s_a
	bsf		F_res6
	movlw	d'23'
	movwf	T100m_a
	btfss	p_SWIN			; p_SWIN ist bei Taster L
	bcf		F_res6			; clr Flag, wenn Taster gedrueckt
	decfsz	T100m_a, F
	goto	$-3				; 23-mal
	decfsz	T1s_a, F
	goto	$-7				; weiter testen
	btfss	F_res6
	goto	p_SWIN_30up_wait; wenn Taster nicht immer offen war
	goto	hw_off_fast
;*********************************
lowBatt_PD
	bcf		INTCON, T0IE	; TMR0-Int-disable
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	bsf		OPTION_REG,	T0CS; Timer0 als counter
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
	bcf		p_CHARG			; C entladen
Msg_lowBatt
	call	ClrDepLns		; erst LCD-Zeilen loeschen
	movlw	d'20'			; xy-Pos einstellen fuer 1. Zeile
	movwf	CusorX
	movlw	d'1'
	movwf	CusorY
	call	setCusor
;*********************************
	movlw	0x19			; l
	movwf	Zeichen
	call	wrChar
	movlw	0x1A			; o
	movwf	Zeichen
	call	wrChar
	movlw	0x1F			; w
	movwf	Zeichen
	call	wrChar
;*********************************
	movlw	d'15'			; ; xy-Pos einstellen fuer 2. Zeile
	movwf	CusorX
	movlw	d'3'
	movwf	CusorY
	call	setCusor
;*********************************
	movlw	0x16			; b
	movwf	Zeichen
	call	wrChar
	movlw	0x15			; a
	movwf	Zeichen
	call	wrChar
	movlw	0x1D			; t
	movwf	Zeichen
	call	wrChar
	movlw	0x1D			; t
	movwf	Zeichen
	call	wrChar
	movlw	0x20			; .
	movwf	Zeichen
	call	wrChar
; dies war ein 2-Zeiler
	bcf		F_HLB0
	bsf		F_HLB1
;*********************************
; ist mit PC verbunden?
	btfss	p_RX
	goto	lB_Msg_2s_wait
	movlw	d'11'			; mit PC verbunden
	movwf	PCtxtGr
	movlw	HIGH PC_Text_06
	movwf	PC_H
	movlw	LOW PC_Text_06
	movwf	PC_L
	call	ReadTabPC
;*********************************
lB_Msg_2s_wait				; die Meldung min. 2 sec anzeigen
	clrf	T100m_a			; clr lowBat-Anzeige-Zeit
	call	C_Entladung		; bis unter 0,425V entladen
	movf	T100m_a, W
	sublw	d'19'			; 2s = (19 + 1) * 100ms
	btfsc	STATUS, C
	goto	$-3				; Ergebnis ist nicht negativ
	goto	hw_off_fast
;*********************************
hw_off						; mit Msg. fuer hw_off (......)
	bcf		INTCON, GIE		; disable global Ints...
	call	ClrDepLns		; erst LCD-Zeilen loeschen
	movlw	d'7'			; xy-Pos einstellen fuer 2. Zeile
	movwf	CusorX
	movlw	d'2'
	movwf	CusorY
	call	setCusor
;*********************************
	movlw	0x20			; .
	movwf	Zeichen
	call	wrChar
	movlw	0x20			; .
	movwf	Zeichen
	call	wrChar
	movlw	0x20			; .
	movwf	Zeichen
	call	wrChar
	movlw	0x20			; .
	movwf	Zeichen
	call	wrChar
	movlw	0x20			; .
	movwf	Zeichen
	call	wrChar
	movlw	0x20			; .
	movwf	Zeichen
	call	wrChar
;*********************************
; ist mit PC verbunden?
	btfss	p_RX
	goto	hw_off_fast
	movlw	d'8'			; mit PC verbunden
	movwf	PCtxtGr
	movlw	HIGH PC_Text_07
	movwf	PC_H
	movlw	LOW PC_Text_07
	movwf	PC_L
	call	ReadTabPC
;*********************************
hw_off_fast					; ohne Msg.
	bcf		INTCON, GIE		; disable global Ints
	call	C_Entladung		; bis unter 0,425V entladen
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	btfss	TXSTA, TRMT		; Test, ob SerBuffer leer
	goto	$-1	
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
	bcf		p_res			; wg. LCD
	bcf		p_SWOUT			; switch off
	goto	$-2
;**********************************************************************
; Unterprogramme
Uref_2_7625V				; Vref = X*Vdd/24 (Bits 3..0)
	movlw	0xAD			; & enable & range
	goto	setUref
Uref_0_425V					; Vref = X*Vdd/24 (Bits 3..0)
	movlw	0xA2			; & enable & range
setUref
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	movwf	VRCON			; set SpgReferenz
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
	goto	$+1				; 10 microSec warten
	goto	$+1
	call	retUref
retUref
	return
;*********************************
C_Entladung
	call	Uref_0_425V		; SpgRef auf 0,425V
; discharge C
	bcf		p_CHARG			; RB5 auf L
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
	bcf		TRISB, 5		; RB5 auf Low_Z
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
Uc_zu_viel					; Uc am Comp. C1
	btfss	CMCON, C1OUT
	goto	Uc_zu_viel		; Uc ist groesser als 0,425V
	return					; return mit RB5 = Low_Z
;*********************************
L_nibble					; nur die unteren 4 Bits
	andlw	0x0F
	movwf	Zeichen
	call	wrChar			; zum LCD
	return
;*********************************
L_nibble_ascii				; nur die unteren 4 Bits
	andlw	0x0F
	addlw	0x30			; addiere fuer ascii-Zeichensatz
	call	writeSer		; zum PC
	return
;*********************************
writeSer					; async.transmit 1 Byte (ist im wreg)
	bsf		STATUS, RP0 	; ---------------------------Select Bank1
EmptyTest
	btfss	TXSTA, TRMT		; Test, ob SerBuffer leer
	goto	EmptyTest
	bcf		STATUS, RP0 	; ---------------------------Select Bank0
	movwf	TXREG			; transmit 1 Byte im WREG
	return
;*********************************
; Binary to BCD conversion routine
; 16 bit number to convert is in C_Val_H, C_Val_L
; result is set in BCDval HML 
HexBCD
	movlw	0x10
	movwf	count
	clrf	BCDvalH
	clrf	BCDvalM
	clrf	BCDvalL
	bcf		STATUS,C
loop16
	rlf		C_Val_L, F
	rlf		C_Val_H, F
	rlf		BCDvalL, F
	rlf		BCDvalM, F
	rlf		BCDvalH, F
	decf	count, F
	btfsc	STATUS, Z
	return
adjDEC
	movlw	BCDvalL
	movwf	FSR
	call	adjBCD
	movlw	BCDvalM
	movwf	FSR
	call	adjBCD
	movlw	BCDvalH
	movwf	FSR
	call	adjBCD
	goto	loop16
adjBCD
	movlw	0x03
	addwf	INDF, W
	movwf	Temp_a
	btfsc	Temp_a, 3
	movwf	INDF
	movlw	0x30
	addwf	INDF, W
	movwf	Temp_a
	btfsc	Temp_a, 7
	movwf	INDF
	return
;*********************************
ClrDepLns
	movf	Flags0, W
	andlw	b'00000011'
	xorlw	0x01
	btfsc	STATUS, Z
	call	clr1DZ
	movf	Flags0, W
	andlw	b'00000011'
	xorlw	0x02
	btfsc	STATUS, Z
	call	clr2DZ
	call	clr3DZ
	return
;*********************************
clr1DZ
	movlw	0x02
	movwf	CusorY			; y-start-pos
	movwf	Temp_a			; Anzahl orig-Zeilen
	goto	CusorX0
clr2DZ
	movlw	0x01
	movwf	CusorY			; y-start-pos
	movlw	0x04
	movwf	Temp_a			; Anzahl orig-Zeilen
	goto	CusorX0
clr3DZ
	clrf	CusorY			; y-start-pos
	movlw	0x06
	movwf	Temp_a			; Anzahl orig-Zeilen
CusorX0
	clrf	CusorX
StClr
	call	setCusor
	movlw	d'84'			; LCD-Spalten
	movwf	CusorX
x_loop
	clrw
	call	wrData
	decfsz	CusorX, F
	goto	x_loop
	incf	CusorY, F
	decfsz	Temp_a, F
	goto	StClr
	return					; from Clr3DZ  evtl. decf Cusor, F
;*********************************
setCusor
;xtest
	movlw	d'84'			; LCD-Spalten
	subwf	CusorX, W
	btfss	STATUS, C
	goto	ytest
	movlw	0x53
	movwf	CusorX
ytest
	movlw	0x06
	subwf	CusorY, W
	btfss	STATUS, C
	goto	orcusor
	movlw	0x05
	movwf	CusorY
orcusor
	movlw	0x80			; setCusor x
	iorwf	CusorX, W
	call	wrComm
	movlw	0x40			; setCusor y
	iorwf	CusorY, W
	call	wrComm
	return					; from setCusor
;*********************************
wrComm
	bcf		p_Dc
	goto	writeIt
wrData
	bsf		p_Dc
writeIt
	bsf		F_res7			; TMR1-Ints nicht verarbeiten
    movwf   LCD_Byte
	bcf		p_ce
	bsf		STATUS, C		; set end-of-loop-flag
	rlf		LCD_Byte, F		; hoehstes Bit in carry
SendLoop
	bcf		p_CLK			; clock auf L
	bcf		p_DIN			; Data auf L
	btfsc	STATUS, C		; check rausgeschobenes Bit
	bsf		p_DIN			; ggf. auf H
	bsf		p_CLK			; mit pos. Flanke uebernehmen
	bcf		STATUS, C		; clr C
	rlf		LCD_Byte, F		; naechtes (hoehstes) Bit in carry
	movf	LCD_Byte, F		; ggf. ZERO setzen
	btfss	STATUS, Z		; bei LCD_Byte = 0 > fertig
	goto	SendLoop
	bsf		p_ce
	bcf		F_res7			; TMR1-Ints wieder verarbeiten
	return					; from wrData / wrComm
;*********************************
wrChar  ; Wert ist in Zeichen, auf Position aus CusorX, CusorY
	movlw	0x21			; !!! testen, ob groesser als d'33'(Zeichenanzahl)
	subwf	Zeichen, W
	btfsc	STATUS, C
	return					; weil groesser als d'33'
	call	setCusor
; Zeilen-Anzahl (Zeichenhoehe in Bytes, hier 2) setzen
	movlw	0x02			; obere und untere
	movwf	OU_Cnt			; Zeile schreiben
; offset auf Zeichengroesse in Bytes (hier 20) multiplizieren
	movf	Zeichen, W		; Zeichen in W zum multiplizieren
; W mit Konstante-20-Multiplier-Routine
    movwf   L_byte
    clrf    H_byte
    bcf     STATUS, C       ; Clr the carry zum rotieren
    rlf     L_byte, F
    rlf     H_byte, F
    bcf     STATUS, C       ; Clr the carry zum rotieren
    rlf     L_byte, F
    rlf     H_byte, F       ; es ist mit 4 multipl. worden
; dieses Ergebnis zwischenspeichern
    movf    H_byte, W
    movwf   Temp_m
    movf    L_byte, W       ; lower Byte bleibt im W
; nochmal mit 4 multipl.
    bcf     STATUS, C       ; Clr the carry zum rotieren
    rlf     L_byte, F
    rlf     H_byte, F
    bcf     STATUS, C       ; Clr the carry zum rotieren
    rlf     L_byte, F
    rlf     H_byte, F       ; es ist mit 16 multipl. worden
; zusammenaddieren
    addwf   L_byte, F
    btfsc   STATUS, C
    incf    H_byte, F
    movf    Temp_m, W
    addwf   H_byte, F
; ZeichenTab-Basis holen und Zeichen-Offset addieren
	movlw	HIGH ZeichenTab	; high 5 bits of address in W
    addwf   H_byte, F       ; addiere Zeichen-Offset Result in H_byte
    movlw   LOW ZeichenTab  ; low 8 bits of address in W
    addwf   L_byte, F       ; addiere Zeichen-Offset Result in L_byte
    btfsc   STATUS, C
    incf    H_byte, F       ; ggf. inc.
nextCols_
    movlw   0x0A            ; 10-mal die column schreiben
	movwf	ColmCnt
nextCol ; eine column schreiben
    movf    H_byte, W
    movwf   PCLATH
    movf    L_byte, W
    call    GetLcdChr
	call	wrData
    incf    L_byte, F
    btfsc   STATUS, Z
    incf    H_byte, F       ; ggf. inc.
	decfsz	ColmCnt, F
	goto	nextCol
; Leerspalte schreiben
	clrw
	call	wrData
	decfsz	OU_Cnt, F		; Zeilen fertig ?
	goto	NextPos
; Cusor auf oben-links fuer naechstes Zeichen
	decf	CusorY, F		; CusorY restaurieren
	movlw	0x0C			; CusorX auf next Text-Pos.
	addwf	CusorX, F
	return					; from wrChar
NextPos
    incf    CusorY, F
    call    setCusor
    goto    nextCols_
;**********************************************************************
ReadTabPC
	clrf	PCtxtPos
Next_1
	movf	PC_H, W
	movwf	PCLATH
	movf	PC_L, W
	addwf	PCtxtPos, W
	btfsc	STATUS, C
	incf	PCLATH, F
	call	GetPC_Txt
	call	writeSer
	incf	PCtxtPos, F
	decfsz	PCtxtGr, F
	goto	Next_1
	return
;*********************************
; PC-Texte-Tabelle
	org			0x052F
GetPC_Txt
	movwf	PCL
PC_Text_01					; on
	dt	"on", 0x0A, 0x0D
PC_Text_02					; C>Cmax
	dt	"C > Cmax", 0x0A, 0x0D
PC_Text_03					; C<Cmin
	dt	"C < Cmin", 0x0A, 0x0D
PC_Text_04					; auto-(power-off)
	dt	"auto-"
PC_Text_05					; power-off
	dt	"power-off", 0x0A, 0x0D
PC_Text_06					; Low Batt.
	dt	"Low Batt.", 0x0A, 0x0D
PC_Text_07					; ......
	dt	"......", 0x0A, 0x0D
;**********************************************************************
; ZeichenSatzTabelle
	org			0x056B
GetLcdChr
	movwf	PCL
ZeichenTab
    dt      0xF8,0xFC,0x06,0x03,0x03,0x03,0x03,0x06,0xFC,0xF8	;0
    dt      0x0F,0x1F,0x30,0x60,0x60,0x60,0x60,0x30,0x1F,0x0F

    dt      0x00,0x00,0x10,0x18,0x0C,0xFE,0xFF,0x00,0x00,0x00	;1
    dt      0x00,0x00,0x40,0x60,0x60,0x7F,0x7F,0x60,0x60,0x40

    dt      0x08,0x0C,0x06,0x03,0x03,0x03,0x03,0x86,0xFC,0x78	;2
    dt      0x40,0x60,0x70,0x78,0x6C,0x66,0x63,0x61,0x60,0x60

    dt      0x03,0x03,0x03,0x03,0x23,0x73,0xDB,0x8F,0x07,0x03	;3
    dt      0x08,0x18,0x30,0x60,0x60,0x60,0x60,0x31,0x1F,0x0E

    dt      0x80,0xC0,0x60,0x30,0x18,0x0C,0xFE,0xFF,0x00,0x00	;4
    dt      0x07,0x07,0x06,0x06,0x06,0x06,0x7F,0x7F,0x06,0x06

    dt      0x7F,0x7F,0x63,0x63,0x63,0x63,0x63,0xC3,0x83,0x03	;5
    dt      0x08,0x18,0x30,0x60,0x60,0x60,0x60,0x30,0x1F,0x0F

    dt      0xE0,0xF0,0x98,0xCC,0xC6,0xC3,0xC1,0x80,0x80,0x00	;6
    dt      0x0F,0x3F,0x31,0x60,0x60,0x60,0x60,0x31,0x3F,0x0F

    dt      0x03,0x03,0x03,0x03,0x83,0xC3,0x63,0x33,0x1F,0x0F	;7
    dt      0x00,0x00,0x00,0x7F,0x7F,0x00,0x00,0x00,0x00,0x00

    dt      0x18,0x3C,0xE6,0xC3,0xC3,0xC3,0xC3,0xE6,0x3C,0x18	;8
    dt      0x0E,0x3F,0x31,0x60,0x60,0x60,0x60,0x31,0x3F,0x0E

    dt      0x38,0xFE,0xC6,0x83,0x83,0x83,0x83,0xC6,0xFE,0xF8	;9
    dt      0x00,0x00,0x00,0x41,0x61,0x31,0x19,0x0C,0x07,0x03

    dt      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;
    dt      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00

    dt      0x00,0xC0,0xC0,0x00,0x00,0x00,0x00,0xC0,0xC0,0x00	;
    dt      0x00,0xFF,0xFF,0x18,0x30,0x30,0x18,0x1F,0x0F,0x00

    dt      0xFF,0xFF,0xC3,0xC3,0xC3,0xC3,0xC3,0xC3,0x03,0x00	;F
    dt      0x7F,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00

    dt      0xE0,0xF8,0x1C,0x0E,0x07,0x03,0x03,0x07,0x0E,0x0C	;C
    dt      0x03,0x0F,0x1C,0x38,0x70,0x60,0x60,0x70,0x38,0x18

    dt      0x00,0x00,0x08,0x18,0x30,0x60,0xC0,0x80,0x00,0x00	;>
    dt      0x00,0x00,0x08,0x0C,0x06,0x03,0x01,0x00,0x00,0x00

    dt      0x00,0x00,0x80,0xC0,0x60,0x30,0x18,0x08,0x00,0x00	;<
    dt      0x00,0x00,0x00,0x01,0x03,0x06,0x0C,0x08,0x00,0x00

    dt      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;a_tief
    dt      0x00,0x00,0x3C,0x66,0xC3,0x81,0xC3,0x7E,0xFE,0x00

    dt      0x00,0x00,0x00,0x00,0xC0,0xC0,0x00,0x00,0x00,0x00	;i_tief
    dt      0x00,0x00,0x00,0x00,0xFE,0xFE,0x00,0x00,0x00,0x00

    dt      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;m_tief
    dt      0xFF,0xFE,0x01,0x01,0xFE,0xFE,0x01,0x01,0xFE,0xFE

    dt      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;n_tief
    dt      0xFF,0xFE,0x01,0x01,0xFE,0xFE,0x00,0x00,0x00,0x00

    dt      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;x_tief
    dt      0x81,0xC3,0x66,0x3C,0x18,0x3C,0x66,0xC3,0x81,0x00

    dt      0x00,0x80,0xC0,0xC0,0xC0,0xC0,0x80,0xC0,0xC0,0x00	;a
    dt      0x0E,0x3F,0x71,0x60,0x60,0x71,0x3B,0x7F,0x7F,0x60

    dt      0x00,0xFF,0xFF,0x80,0xC0,0xC0,0xC0,0xC0,0x80,0x00	;b
    dt      0x00,0x7F,0x7F,0x31,0x71,0x60,0x60,0x71,0x3F,0x0E

    dt      0x00,0x00,0x80,0xC0,0xC0,0xC0,0xC0,0x80,0x00,0x00	;e
    dt      0x00,0x0E,0x3F,0x75,0x64,0x64,0x75,0x37,0x06,0x00

    dt      0x00,0x00,0x60,0x60,0xFE,0xFF,0x63,0x62,0x00,0x00	;f
    dt      0x00,0x00,0x00,0x00,0x7F,0x7F,0x00,0x00,0x00,0x00

    dt      0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0x00,0x00	;l
    dt      0x00,0x00,0x00,0x00,0x00,0x00,0x3F,0x7F,0x60,0x40

    dt      0x00,0x00,0x80,0xC0,0xC0,0xC0,0xC0,0x80,0x00,0x00	;o
    dt      0x00,0x1F,0x3F,0x71,0x60,0x60,0x71,0x3F,0x1F,0x00

    dt      0x00,0x00,0xC0,0xC0,0x80,0xC0,0xC0,0xC0,0x80,0x00	;p
    dt      0x00,0x00,0xFF,0xFF,0x19,0x30,0x30,0x39,0x1F,0x0F

    dt      0x00,0xC0,0xC0,0x80,0xC0,0xC0,0x80,0x80,0x00,0x00	;r
    dt      0x00,0x7F,0x7F,0x01,0x00,0x00,0x01,0x01,0x00,0x00

    dt      0x00,0x00,0x60,0x60,0xFF,0xFF,0x60,0x60,0x00,0x00	;t
    dt      0x00,0x00,0x00,0x00,0x3F,0x7F,0x60,0x40,0x00,0x00

    dt      0x00,0xC0,0xC0,0x00,0x00,0x00,0x00,0x00,0xC0,0xC0	;u
    dt      0x00,0x1F,0x3F,0x70,0x60,0x60,0x60,0x30,0x7F,0x7F

    dt      0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80	;w
    dt      0x07,0x1F,0x7C,0x70,0x1C,0x1C,0x70,0x7C,0x1F,0x07

    dt      0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;.
    dt      0x70,0x70,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00

			END                       ; directive 'end of program'
